//***************************************************************************
//  File........: lcd_lib.c
//
//  Author(s)...: Pashgan    chipenable.ru
//
//   .....: andries5
//
//  Target(s)...: ATMega...
//
//  Compiler....: IAR EWAAVR 5.11a
//
//  Description.:    
//
//  Data........: 28.10.09  
//
//***************************************************************************
#include "lcd_lib.h"
#include <util/delay.h>

//    
#define setbit_0(reg, bit) (reg&=~(1<<bit))
#define setbit_1(reg, bit) (reg|= (1<<bit))  
#define FLAG_BF 7

#ifdef LCD_low
	char LCD_DD=0x0F;
	char LCD_D4=0;	//define MCU pin connected to LCD D0
	char LCD_D5=1;	//define MCU pin connected to LCD D1
	char LCD_D6=2;	//define MCU pin connected to LCD D2
	char LCD_D7=3;	//define MCU pin connected to LCD D3
#else
	char LCD_DD=0xF0;
	char LCD_D4=4;	//define MCU pin connected to LCD D4
	char LCD_D5=5;	//define MCU pin connected to LCD D5
	char LCD_D6=6;	//define MCU pin connected to LCD D6
	char LCD_D7=7;	//define MCU pin connected to LCD D7
#endif


//  
//#define _delay_us(us)     __delay_cycles((F_CPU / 1000000) * (us));
//#define _delay_ms(ms)     __delay_cycles((F_CPU / 1000) * (ms));

// 
//#pragma inline = forced


inline unsigned char __swap_nibbles(unsigned char data)
{
  asm volatile("swap %0" : "=r" (data) : "0" (data));
  return data;
}


void Lcd_CommonFunc(unsigned char data)
{
 
  unsigned char tmp; 
	tmp = LDP & ~(LCD_DD);
	#ifdef LCD_low
		tmp= __swap_nibbles(tmp);
		tmp |= (data & ~(LCD_DD));
		tmp= __swap_nibbles(tmp);
	#else
		tmp |= (data & LCD_DD);
	#endif
  LDP = tmp;        //   

  setbit_1(LCP, EN);            
  _delay_ms(1);
  setbit_0(LCP, EN);    

  tmp  = LDP & ~(LCD_DD);
	#ifdef LCD_low
		tmp |= (data & LCD_DD);
	#else
		data = __swap_nibbles(data);
		
		tmp |= (data & LCD_DD);
	#endif 
  LDP = tmp;        //   
  setbit_1(LCP, EN);            
  _delay_ms(1);
  setbit_0(LCP, EN); 
	_delay_ms(1);

}

//   lcd
void Lcd_Wait(void)
{
#ifdef CHECK_FLAG_BF
  unsigned char data;
  LDDR &= ~(LCD_DD);            //   
  LDP |= LCD_DD;            // pull-up 
  setbit_1(LCP, RW);         //RW  1   lcd
  setbit_0(LCP, RS);    //RS  0 
  do{
    setbit_1(LCP, EN);    
    _delay_us(2);
    data = LDPIN & LCD_DD;      //   
    setbit_0(LCP, EN);
    data = __swap_nibbles(data);
    setbit_1(LCP, EN);    
    _delay_us(2);
    data |= LDPIN & LCD_DD;      //   
    setbit_0(LCP, EN);
    #ifndef LCD_low
			data = __swap_nibbles(data);
		#endif
  }while((data & (1<<FLAG_BF))!= 0 );
  setbit_0(LCP, RW);
  LDDR |= LCD_DD; 
#else
  _delay_us(40);
#endif  
}

//   
void Lcd_WriteCom(unsigned char data)
{
  Lcd_Wait();
	setbit_0(LCP, RS);    // RS  0 - 
  Lcd_CommonFunc(data);
}

//  
void Lcd_WriteData(unsigned char data)
{
  Lcd_Wait();
	setbit_1(LCP, RS);            // RS  1 - 
  Lcd_CommonFunc(data);
}

// 
void Lcd_Init(void)
{
	_delay_ms(40);
	LDP &= ~(LCD_DD);
	LCP&= ~((1<<RS)|(1<<RW)|(1<<EN));
	LDDR|=LCD_DD;
	LCDR|=1<<EN|1<<RW|1<<RS;
	Lcd_WriteCom(0x03);
	_delay_ms(5);
	Lcd_WriteCom(0x03);
	Lcd_WriteCom(0x02);
	Lcd_WriteCom(0x28); //4-  , 2 
	Lcd_WriteCom(0x0F);  //0b00001111 - , ,  
 	Lcd_WriteCom(0x01);  //0b00000001 -  
  Lcd_WriteCom(0x0C);  //0b00000110 -   ,  
}

//     
void Lcd_StrF(prog_char *str)
{
  unsigned char data;            
  while (*str)
  {
    data = *str++;
    Lcd_WriteData(data);
  }
}

//    
void Lcd_Str(char *str)
{
  unsigned char data;
  while (*str)
  {
    data = *str++;
    Lcd_WriteData(data);
  }
}

void Lcd_Goto(char x,char y)
{
	Lcd_WriteCom(((((y)& 1)*0x40)+((x)& 7))|128);
}

void Lcd_Clr(void)
{
	Lcd_WriteCom(0x01);
}